@@ -1,5 +1,3 @@
-#!/usr/bin/python
-#
 # Copyright 2010 Dan Smith <dsmith@danplanet.com>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -19,11 +17,21 @@
 #
 # Example definitions:
 #
+#  bit  foo[8];  /* Eight single bit values                 */
 #  u8   foo;     /* Unsigned 8-bit value                    */
 #  u16  foo;     /* Unsigned 16-bit value                   */
 #  ul16 foo;     /* Unsigned 16-bit value (LE)              */
 #  u24  foo;     /* Unsigned 24-bit value                   */
+#  ul24 foo;     /* Unsigned 24-bit value (LE)              */
 #  u32  foo;     /* Unsigned 32-bit value                   */
+#  ul32 foo;     /* Unsigned 32-bit value (LE)              */
+#  i8   foo;     /* Signed 8-bit value                      */
+#  i16  foo;     /* Signed 16-bit value                     */
+#  il16 foo;     /* Signed 16-bit value (LE)                */
+#  i24  foo;     /* Signed 24-bit value                     */
+#  il24 foo;     /* Signed 24-bit value (LE)                */
+#  i32  foo;     /* Signed 32-bit value                     */
+#  il32 foo;     /* Signed 32-bit value (LE)                */
 #  char foo;     /* Character (single-byte                  */
 #  lbcd foo;     /* BCD-encoded byte (LE)                   */
 #  bbcd foo;     /* BCD-encoded byte (BE)                   */
@@ -55,6 +63,10 @@ import os
 from chirp import bitwise_grammar
 from chirp.memmap import MemoryMap
 
+class ParseError(Exception):
+    """Indicates an error parsing a definition"""
+    pass
+
 def format_binary(nbits, value, pad=8):
     s = ""
     for i in range(0, nbits):
@@ -138,6 +150,10 @@ class DataElement:
     def set_raw(self, data):
         self._data[self._offset] = data[:self._size]
 
+    def __getattr__(self, name):
+        raise AttributeError("Unknown attribute %s in %s" % (name,
+                                                             self.__class__))
+
     def __repr__(self):
         return "(%s:%i bytes @ %04x)" % (self.__class__.__name__,
                                          self._size,
@@ -149,7 +165,7 @@ class arrayDataElement(DataElement):
             return "%i:[(%i)]" % (len(self.__items), int(self))
 
         if isinstance(self.__items[0], charDataElement):
-            return "%i:[(%s)]" % (len(self.__items), str(self))
+            return "%i:[(%s)]" % (len(self.__items), repr(str(self))[1:-1])
 
         s = "%i:[" % len(self.__items)
         s += ",".join([repr(item) for item in self.__items])
@@ -166,6 +182,9 @@ class arrayDataElement(DataElement):
     def get_value(self):
         return list(self.__items)
 
+    def get_raw(self):
+        return "".join([item.get_raw() for item in self.__items])
+
     def __setitem__(self, index, val):
         self.__items[index].set_value(val)
 
@@ -182,18 +201,16 @@ class arrayDataElement(DataElement):
             return str(self.__items)
 
     def __int__(self):
-        if isinstance(self.__items[0], bbcdDataElement):
+        if isinstance(self.__items[0], bcdDataElement):
             val = 0
-            for i in self.__items:
+            if isinstance(self.__items[0], bbcdDataElement):
+                items = self.__items
+            else:
+                items = reversed(self.__items)
+            for i in items:
                 tens, ones = i.get_value()
                 val = (val * 100) + (tens * 10) + ones
             return val
-        elif isinstance(self.__items[0], lbcdDataElement):
-            val = 0
-            for i in reversed(self.__items):
-                ones, tens = i.get_value()
-                val = (val * 100) + (tens * 10) + ones
-            return val
         else:
             raise ValueError("Cannot coerce this to int")
 
@@ -210,16 +227,19 @@ class arrayDataElement(DataElement):
             i.set_value(twodigits)
 
     def __set_value_char(self, value):
+        if len(value) != len(self.__items):
+            raise ValueError("String expects exactly %i characters" %
+                             len(self.__items))
         for i in range(0, len(self.__items)):
             self.__items[i].set_value(value[i])
 
     def set_value(self, value):
         if isinstance(self.__items[0], bbcdDataElement):
-            self.__set_value_bbcd(value)
+            self.__set_value_bbcd(int(value))
         elif isinstance(self.__items[0], lbcdDataElement):
-            self.__set_value_lbcd(value)
+            self.__set_value_lbcd(int(value))
         elif isinstance(self.__items[0], charDataElement):
-            self.__set_value_char(value)
+            self.__set_value_char(str(value))
         elif len(value) != len(self.__items):
             raise ValueError("Array cardinality mismatch")
         else:
@@ -237,6 +257,12 @@ class arrayDataElement(DataElement):
     def __iter__(self):
         return iter(self.__items)
 
+    def items(self):
+        index = 0
+        for item in self.__items:
+            yield (str(index), item)
+            index += 1
+
     def size(self):
         size = 0
         for i in self.__items:
@@ -367,7 +393,7 @@ class u8DataElement(intDataElement):
         return ord(data)
 
     def set_value(self, value):
-        self._data[self._offset] = (value & 0xFF)
+        self._data[self._offset] = (int(value) & 0xFF)
 
 class u16DataElement(intDataElement):
     _size = 2
@@ -378,19 +404,32 @@ class u16DataElement(intDataElement):
 
     def set_value(self, value):
         self._data[self._offset] = struct.pack(self._endianess + "H",
-                                               value & 0xFFFF)
+                                               int(value) & 0xFFFF)
 
 class ul16DataElement(u16DataElement):
     _endianess = "<"
-
+    
 class u24DataElement(intDataElement):
     _size = 3
+    _endianess = ">"
 
     def _get_value(self, data):
-        return struct.unpack(">I", "\x00" + data)[0]
+        pre = self._endianess == ">" and "\x00" or ""
+        post = self._endianess == "<" and "\x00" or ""
+        return struct.unpack(self._endianess + "I", pre+data+post)[0]
 
     def set_value(self, value):
-        self._data[self._offset] = struct.pack(">I", value & 0xFFFFFFFF)[1:]
+        if self._endianess == "<":
+            start = 0
+            end = 3
+        else:
+            start = 1
+            end = 4
+        self._data[self._offset] = struct.pack(self._endianess + "I",
+                                               int(value) & 0xFFFFFFFF)[start:end]
+
+class ul24DataElement(u24DataElement):
+    _endianess = "<"
 
 class u32DataElement(intDataElement):
     _size = 4
@@ -401,54 +440,126 @@ class u32DataElement(intDataElement):
 
     def set_value(self, value):
         self._data[self._offset] = struct.pack(self._endianess + "I",
-                                               value & 0xFFFFFFFF)
+                                               int(value) & 0xFFFFFFFF)
 
 class ul32DataElement(u32DataElement):
     _endianess = "<"
 
+class i8DataElement(u8DataElement):
+    _size = 1
+
+    def _get_value(self, data):
+        return struct.unpack("b", data)[0]
+
+    def set_value(self, value):
+        self._data[self._offset] = struct.pack("b", int(value) )
+        
+class i16DataElement(intDataElement):
+    _size = 2
+    _endianess = ">"
+
+    def _get_value(self, data):
+        return struct.unpack(self._endianess + "h", data)[0]
+
+    def set_value(self, value):
+        self._data[self._offset] = struct.pack(self._endianess + "h",
+                                               int(value) )
+
+class il16DataElement(i16DataElement):
+    _endianess = "<"
+
+class i24DataElement(intDataElement):
+    _size = 3
+    _endianess = ">"
+
+    def _get_value(self, data):
+        pre = self._endianess == ">" and "\x00" or ""
+        post = self._endianess == "<" and "\x00" or ""
+        return struct.unpack(self._endianess + "i", pre+data+post)[0]
+
+    def set_value(self, value):
+        if self._endianess == "<":
+            start = 0
+            end = 3
+        else:
+            start = 1
+            end = 4
+        self._data[self._offset] = struct.pack(self._endianess + "i",
+                                               int(value) )[start:end]
+
+class il24DataElement(i24DataElement):
+    _endianess = "<"
+
+class i32DataElement(intDataElement):
+    _size = 4
+    _endianess = ">"
+
+    def _get_value(self, data):
+        return struct.unpack(self._endianess + "i", data)[0]
+
+    def set_value(self, value):
+        self._data[self._offset] = struct.pack(self._endianess + "i",
+                                               int(value) )
+
+class il32DataElement(i32DataElement):
+    _endianess = "<"
+
 class charDataElement(DataElement):
     _size = 1
 
+    def __str__(self):
+        return str(self.get_value())
+
+    def __int__(self):
+        return ord(self.get_value())
+
     def _get_value(self, data):
         return data
 
     def set_value(self, value):
-        self._data[self._offset] = value
+        self._data[self._offset] = str(value)
 
 class bcdDataElement(DataElement):
+    def __int__(self):
+        tens, ones = self.get_value()
+        return (tens * 10) + ones
+
     def set_bits(self, mask):
-        self._data[self._offset] = ord(self._data[self._offset]) | mask
+        self._data[self._offset] = ord(self._data[self._offset]) | int(mask)
 
     def clr_bits(self, mask):
-        self._data[self._offset] = ord(self._data[self._offset]) & ~mask
+        self._data[self._offset] = ord(self._data[self._offset]) & ~int(mask)
 
-class lbcdDataElement(bcdDataElement):
-    _size = 1
+    def get_bits(self, mask):
+        return ord(self._data[self._offset]) & int(mask)
 
-    def _get_value(self, data):
-        a = (ord(data) & 0xF0) >> 4
-        b = ord(data) & 0x0F
-        return (b, a)
+    def set_raw(self, data):
+        if isinstance(data, int):
+            self._data[self._offset] = data & 0xFF
+        elif isinstance(data, str):
+            self._data[self._offset] = data[0]
+        else:
+            raise TypeError("Unable to set bcdDataElement from type %s" %
+                            type(data))
 
     def set_value(self, value):
-        value = int("%02i" % value, 16)
-        self._data[self._offset] = value
-
-class bbcdDataElement(bcdDataElement):
-    _size = 1
+        self._data[self._offset] = int("%02i" % value, 16)
 
     def _get_value(self, data):
         a = (ord(data) & 0xF0) >> 4
         b = ord(data) & 0x0F
         return (a, b)
 
-    def set_value(self, value):
-        self._data[self._offset] = int("%02i" % value, 16)
+class lbcdDataElement(bcdDataElement):
+    _size = 1
+
+class bbcdDataElement(bcdDataElement):
+    _size = 1
 
 class bitDataElement(intDataElement):
     _nbits = 0
     _shift = 0
-    _subgen = None
+    _subgen = u8DataElement # Default to a byte
 
     def __repr__(self):
         fmt = "0x%%0%iX (%%sb)" % (self._size * 2)
@@ -476,7 +587,7 @@ class bitDataElement(intDataElement):
         #print "mask: %04x" % mask
         #print "valu: %04x" % value
 
-        value = ((value << (self._shift-self._nbits)) & mask) | data
+        value = ((int(value) << (self._shift-self._nbits)) & mask) | data
         self._subgen(self._data, self._offset).set_value(value)
         
     def size(self):
@@ -533,7 +644,10 @@ class structDataElement(DataElement):
             self._keys.append(key)
 
     def __getattr__(self, name):
-        return self._generators[name]
+        try:
+            return self._generators[name]
+        except KeyError:
+            raise AttributeError("No attribute %s in struct" % name)
 
     def __setattr__(self, name, value):
         if not self.__dict__.has_key("_structDataElement__init"):
@@ -563,6 +677,14 @@ class structDataElement(DataElement):
             raise ValueError("Struct size mismatch during set_raw()")
         self._data[self._offset] = buffer
 
+    def __iter__(self):
+        for item in self._generators.values():
+            yield item
+
+    def items(self):
+        for key in self._keys:
+            yield key, self._generators[key]
+
 class Processor:
 
     _types = {
@@ -570,8 +692,15 @@ class Processor:
         "u16"  : u16DataElement,
         "ul16" : ul16DataElement,
         "u24"  : u24DataElement,
+        "ul24" : ul24DataElement,
         "u32"  : u32DataElement,
         "ul32" : ul32DataElement,
+        "i8"   : i8DataElement,
+        "i16"  : i16DataElement,
+        "il16" : il16DataElement,
+        "i24"  : i24DataElement,
+        "il24" : il24DataElement,
+        "i32"  : i32DataElement,
         "char" : charDataElement,
         "lbcd" : lbcdDataElement,
         "bbcd" : bbcdDataElement,
@@ -581,6 +710,7 @@ class Processor:
         self._data = data
         self._offset = offset
         self._obj = None
+        self._user_types = {}
 
     def do_symbol(self, symdef, gen):
         name = symdef[1]
@@ -610,6 +740,16 @@ class Processor:
 
         return bytes
 
+    def do_bitarray(self, i, count):
+        if count % 8 != 0:
+            raise ValueError("bit array must be divisible by 8.")
+
+        class bitDE(bitDataElement):
+            _nbits = 1
+            _shift = 8 - i % 8
+
+        return bitDE(self._data, self._offset)
+
     def parse_defn(self, defn):
         dtype = defn[0]
 
@@ -629,23 +769,30 @@ class Processor:
             res = arrayDataElement(self._offset)
             size = 0
             for i in range(0, count):
-                gen = self._types[dtype](self._data, self._offset)
+                if dtype == "bit":
+                    gen = self.do_bitarray(i, count)
+                    self._offset += int((i+1) % 8 == 0)
+                else:
+                    gen = self._types[dtype](self._data, self._offset)
+                    self._offset += (gen.size() / 8)
                 res.append(gen)
-                self._offset += (gen.size() / 8)
 
             if count == 1:
                 self._generators[name] = res[0]
             else:
                 self._generators[name] = res
 
-    def parse_struct(self, struct):
+    def parse_struct_decl(self, struct):
         block = struct[:-1]
+        if block[0][0] == "symbol":
+            # This is a pre-defined struct
+            block = self._user_types[block[0][1]]
         deftype = struct[-1]
         if deftype[0] == "array":
             name = deftype[1][0][1]
             count = int(deftype[1][1][1])
         elif deftype[0] == "symbol":
-            name = deftype[0][1]
+            name = deftype[1]
             count = 1
 
         result = arrayDataElement(self._offset)
@@ -663,22 +810,29 @@ class Processor:
         else:
             self._generators[name] = result
 
+    def parse_struct_defn(self, struct):
+        name = struct[0][1]
+        block = struct[1:]
+        self._user_types[name] = block
+
+    def parse_struct(self, struct):
+        if struct[0][0] == "struct_defn":
+            return self.parse_struct_defn(struct[0][1])
+        elif struct [0][0] == "struct_decl":
+            return self.parse_struct_decl(struct[0][1])
+        else:
+            raise Exception("Internal error: What is `%s'?" % struct[0][0])
+
     def parse_directive(self, directive):
         name = directive[0][0]
+        value = directive[0][1][0][1]
         if name == "seekto":
-            offset = directive[0][1][0][1]
-            if offset.startswith("0x"):
-                offset = int(offset[2:], 16)
-            else:
-                offset = int(offset)
             #print "NOTICE: Setting offset to %i (0x%X)" % (offset, offset)
-            self._offset = offset
+            self._offset = int(value, 0)
         elif name == "seek":
-            offset = int(directive[0][1][0][1])
-            self._offset += offset
+            self._offset += int(value, 0)
         elif name == "printoffset":
-            string = directive[0][1][0][1]
-            print "%s: %i (0x%08X)" % (string[1:-1], self._offset, self._offset)
+            print "%s: %i (0x%08X)" % (value[1:-1], self._offset, self._offset)
 
     def parse_block(self, lang):
         for t, d in lang:
@@ -693,7 +847,7 @@ class Processor:
 
     def parse(self, lang):
         self._generators = structDataElement(self._data, self._offset)
-        self.parse_block(lang[0])
+        self.parse_block(lang)
         return self._generators
 
 
@@ -704,6 +858,8 @@ def parse(spec, data, offset=0):
 
 if __name__ == "__main__":
     defn = """
+struct mytype { u8 foo; };
+struct mytype bar;
 struct {
   u8 foo;
   u8 highbit:1,
@@ -711,11 +867,13 @@ struct {
      lowbit:1;
   char string[3];
   bbcd fourdigits[2];
-} mystruct[1];
+} mystruct;
 """
-    data = "\x7F\x81abc\x12\x34"
+    data = "\xab\x7F\x81abc\x12\x34"
     tree = parse(defn, data)
 
+    print repr(tree)
+
     print "Foo %i" % tree.mystruct.foo
     print "Highbit: %i SixZeros: %i: Lowbit: %i" % (tree.mystruct.highbit,
                                                     tree.mystruct.sixzeros,
