@@ -14,9 +14,10 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import re
-from chirp.pyPEG import keyword, parseLine
+from chirp.pyPEG import keyword, parse as pypeg_parse
 
-TYPES = ["u8", "u16", "ul16", "u24", "ul24", "u32", "ul32", "char",
+TYPES = ["bit", "u8", "u16", "ul16", "u24", "ul24", "u32", "ul32",
+         "i8", "i16", "il16", "i24", "il24", "i32", "il32", "char",
          "lbcd", "bbcd"]
 DIRECTIVES = ["seekto", "seek", "printoffset"]
 
@@ -78,4 +79,33 @@ def _language():
     return _block_inner
 
 def parse(data):
-    return parseLine(data, _language, resultSoFar=[]) 
+    lines = data.split("\n")
+    for index, line in enumerate(lines):
+        if '//' in line:
+            lines[index] = line[:line.index('//')]
+
+    class FakeFileInput:
+        """Simulate line-by-line file reading from @data"""
+        line = -1
+
+        def isfirstline(self):
+            return self.line == 0
+
+        def filename(self):
+            return "input"
+
+        def lineno(self):
+            return self.line
+
+        def __iter__(self):
+            return self
+
+        def next(self):
+            self.line += 1
+            try:
+                # Note, FileInput objects keep the newlines
+                return lines[self.line] + "\n"
+            except IndexError:
+                raise StopIteration
+
+    return pypeg_parse(_language, FakeFileInput())
