@@ -32,6 +32,12 @@ TONES = [ 67.0, 69.3, 71.9, 74.4, 77.0, 79.7, 82.5,
           225.7, 229.1, 233.6, 241.8, 250.3, 254.1,
           ]          
 
+TONES_EXTRA = [62.5]
+
+OLD_TONES = list(TONES)
+[OLD_TONES.remove(x) for x in [159.8, 165.5, 171.3, 177.3, 183.5, 189.9,
+                               196.6, 199.5, 206.5, 229.1, 254.1]]
+
 # 104 DTCS Codes
 DTCS_CODES = [
      23,  25,  26,  31,  32,  36,  43,  47,  51,  53,  54,
@@ -46,74 +52,26 @@ DTCS_CODES = [
     731, 732, 734, 743, 754,
      ]
 
-# Some radios have some strange codes
-DTCS_EXTRA_CODES = [ 17, 645 ]
+# 512 Possible DTCS Codes
+ALL_DTCS_CODES = []
+for a in range(0, 8):
+    for b in range(0, 8):
+        for c in range(0, 8):
+            ALL_DTCS_CODES.append((a * 100) + (b * 10) + c)
 
 CROSS_MODES = [
     "Tone->Tone",
+    "DTCS->",
+    "->DTCS",
     "Tone->DTCS",
     "DTCS->Tone",
-    "DTCS->",
     "->Tone",
-    "->DTCS",
     "DTCS->DTCS",
 ]
 
 MODES = ["WFM", "FM", "NFM", "AM", "NAM", "DV", "USB", "LSB", "CW", "RTTY",
          "DIG", "PKT", "NCW", "NCWR", "CWR", "P25", "Auto"]
 
-STD_6M_OFFSETS = [
-    (51620000, 51980000, -500000),
-    (52500000, 52980000, -500000),
-    (53500000, 53980000, -500000),
-    ]
-
-STD_2M_OFFSETS = [
-    (145100000, 145500000, -600000),
-    (146000000, 146400000,  600000),
-    (146600000, 147000000, -600000),
-    (147000000, 147400000,  600000),
-    (147600000, 148000000, -600000),
-    ]
-
-STD_220_OFFSETS = [
-    (223850000, 224980000, -1600000),
-    ]
-
-STD_70CM_OFFSETS = [
-    (440000000, 445000000,  5000000),
-    (447000000, 450000000, -5000000),
-    ]
-
-STD_23CM_OFFSETS = [
-    (1282000000, 1288000000, -12000000),
-    ]
-
-# Standard offsets, indexed by band (wavelength in cm)
-STD_OFFSETS = {
-    600 : STD_6M_OFFSETS,
-    200 : STD_2M_OFFSETS,
-    125 : STD_220_OFFSETS,
-     70 : STD_70CM_OFFSETS,
-     23 : STD_23CM_OFFSETS,
-    }
-
-BAND_TO_MHZ = {
-    600 : (   50000000,   54000000 ),
-    200 : (  144000000,  148000000 ),
-    125 : (  219000000,  225000000 ),
-    70 :  (  420000000,  450000000 ),
-    23 :  ( 1240000000, 1300000000 ),
-}
-
-# NB: This only works for some bands, throws an Exception otherwise
-def freq_to_band(freq):
-    """Returns the band (in cm) for a given frequency"""
-    for band, (lo, hi) in BAND_TO_MHZ.items():
-        if int(freq) > lo and int(freq) < hi:
-            return band
-    raise Exception("No conversion for frequency %i" % freq)
-
 TONE_MODES = [
     "",
     "Tone",
@@ -138,6 +96,52 @@ CHARSET_ALPHANUMERIC = \
     "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz 1234567890"
 CHARSET_ASCII = "".join([chr(x) for x in range(ord(" "), ord("~")+1)])
 
+# http://aprs.org/aprs11/SSIDs.txt
+APRS_SSID = (
+    "0 Your primary station usually fixed and message capable",
+    "1 generic additional station, digi, mobile, wx, etc",
+    "2 generic additional station, digi, mobile, wx, etc",
+    "3 generic additional station, digi, mobile, wx, etc",
+    "4 generic additional station, digi, mobile, wx, etc",
+    "5 Other networks (Dstar, Iphones, Androids, Blackberry's etc)",
+    "6 Special activity, Satellite ops, camping or 6 meters, etc",
+    "7 walkie talkies, HT's or other human portable",
+    "8 boats, sailboats, RV's or second main mobile",
+    "9 Primary Mobile (usually message capable)",
+    "10 internet, Igates, echolink, winlink, AVRS, APRN, etc",
+    "11 balloons, aircraft, spacecraft, etc",
+    "12 APRStt, DTMF, RFID, devices, one-way trackers*, etc",
+    "13 Weather stations",
+    "14 Truckers or generally full time drivers",
+    "15 generic additional station, digi, mobile, wx, etc")
+APRS_POSITION_COMMENT = (
+    "off duty", "en route", "in service", "returning", "committed",
+    "special", "priority", "custom 0", "custom 1", "custom 2", "custom 3",
+    "custom 4", "custom 5", "custom 6", "EMERGENCY")
+# http://aprs.org/symbols/symbolsX.txt
+APRS_SYMBOLS = (
+    "Police/Sheriff", "[reserved]", "Digi", "Phone", "DX Cluster",
+    "HF Gateway", "Small Aircraft", "Mobile Satellite Groundstation",
+    "Wheelchair", "Snowmobile", "Red Cross", "Boy Scouts", "House QTH (VHF)",
+    "X", "Red Dot", "0 in Circle", "1 in Circle", "2 in Circle",
+    "3 in Circle", "4 in Circle", "5 in Circle", "6 in Circle", "7 in Circle",
+    "8 in Circle", "9 in Circle", "Fire", "Campground", "Motorcycle",
+    "Railroad Engine", "Car", "File Server", "Hurricane Future Prediction",
+    "Aid Station", "BBS or PBBS", "Canoe", "[reserved]", "Eyeball",
+    "Tractor/Farm Vehicle", "Grid Square", "Hotel", "TCP/IP", "[reserved]",
+    "School", "PC User", "MacAPRS", "NTS Station", "Balloon", "Police", "TBD",
+    "Recreational Vehicle", "Space Shuttle", "SSTV", "Bus", "ATV",
+    "National WX Service Site", "Helicopter", "Yacht/Sail Boat", "WinAPRS",
+    "Human/Person", "Triangle", "Mail/Postoffice", "Large Aircraft",
+    "WX Station", "Dish Antenna", "Ambulance", "Bicycle",
+    "Incident Command Post", "Dual Garage/Fire Dept", "Horse/Equestrian",
+    "Fire Truck", "Glider", "Hospital", "IOTA", "Jeep", "Truck", "Laptop",
+    "Mic-Repeater", "Node", "Emergency Operations Center", "Rover (dog)",
+    "Grid Square above 128m", "Repeater", "Ship/Power Boat", "Truck Stop",
+    "Truck (18 wheeler)", "Van", "Water Station", "X-APRS", "Yagi at QTH",
+    "TDB", "[reserved]"
+)
+
 def watts_to_dBm(watts):
     """Converts @watts in watts to dBm"""
     return int(10 * math.log10(int(watts * 1000)))
@@ -185,16 +189,28 @@ class PowerLevel:
 
 def parse_freq(freqstr):
     """Parse a frequency string and return the value in integral Hz"""
+    freqstr = freqstr.strip()
+    if freqstr == "":
+        return 0
+    elif freqstr.endswith(" MHz"):
+        return parse_freq(freqstr.split(" ")[0])
+    elif freqstr.endswith(" kHz"):
+        return int(freqstr.split(" ")[0]) * 1000
+
     if "." in freqstr:
         mhz, khz = freqstr.split(".")
+        if mhz == "":
+            mhz = 0
+        khz = khz.ljust(6, "0")
+        if len(khz) > 6:
+            raise ValueError("Invalid kHz value: %s", khz)
+        mhz = int(mhz) * 1000000
+        khz = int(khz)
     else:
-        mhz = freqstr
-        khz = "0"
-    if not mhz.isdigit() and khz.isdigit():
-        raise ValueError("Invalid value")
+        mhz = int(freqstr) * 1000000
+        khz = 0
 
-    # Make kHz exactly six decimal places
-    return int(("%s%-6s" % (mhz, khz)).replace(" ", "0"))
+    return mhz + khz
 
 def format_freq(freq):
     """Format a frequency given in Hz as a string"""
@@ -262,10 +278,10 @@ class Memory:
         self.immutable = []
 
     _valid_map = {
-        "rtone"         : TONES,
-        "ctone"         : TONES,
-        "dtcs"          : DTCS_CODES + DTCS_EXTRA_CODES,
-        "rx_dtcs"       : DTCS_CODES + DTCS_EXTRA_CODES,
+        "rtone"         : TONES + TONES_EXTRA,
+        "ctone"         : TONES + TONES_EXTRA,
+        "dtcs"          : ALL_DTCS_CODES,
+        "rx_dtcs"       : ALL_DTCS_CODES,
         "tmode"         : TONE_MODES,
         "dtcs_polarity" : ["NN", "NR", "RN", "RR"],
         "cross_mode"    : CROSS_MODES,
@@ -525,8 +541,8 @@ class DVMemory(Memory):
         except Exception:
             self.dv_code = 0
 
-class Bank:
-    """Base class for a radio's Bank"""
+class MemoryMapping(object):
+    """Base class for a memory mapping"""
     def __init__(self, model, index, name):
         self._model = model
         self._index = index
@@ -536,75 +552,87 @@ class Bank:
         return self.get_name()
 
     def __repr__(self):
-        return "Bank-%s" % self._index
+        return "%s-%s" % (self.__class__.__name__, self._index)
 
     def get_name(self):
-        """Returns the static or user-adjustable bank name"""
+        """Returns the mapping name"""
         return self._name
 
     def get_index(self):
-        """Returns the immutable bank index (string or int)"""
+        """Returns the immutable index (string or int)"""
         return self._index
 
     def __eq__(self, other):
         return self.get_index() == other.get_index()
 
+class MappingModel(object):
+    """Base class for a memory mapping model"""
+
+    def __init__(self, radio, name):
+        self._radio = radio
+        self._name = name
+
+    def get_name(self):
+        return self._name
+
+    def get_num_mappings(self):
+        """Returns the number of mappings in the model (should be
+        callable without consulting the radio"""
+        raise NotImplementedError()
+
+    def get_mappings(self):
+        """Return a list of mappings"""
+        raise NotImplementedError()
+
+    def add_memory_to_mapping(self, memory, mapping):
+        """Add @memory to @mapping."""
+        raise NotImplementedError()
+
+    def remove_memory_from_mapping(self, memory, mapping):
+        """Remove @memory from @mapping.
+        Shall raise exception if @memory is not in @bank"""
+        raise NotImplementedError()
+
+    def get_mapping_memories(self, mapping):
+        """Return a list of memories in @mapping"""
+        raise NotImplementedError()
+
+    def get_memory_mappings(self, memory):
+        """Return a list of mappings that @memory is in"""
+        raise NotImplementedError()
+
+class Bank(MemoryMapping):
+    """Base class for a radio's Bank"""
+
 class NamedBank(Bank):
     """A bank that can have a name"""
     def set_name(self, name):
         """Changes the user-adjustable bank name"""
         self._name = name
 
-class BankModel:
+class BankModel(MappingModel):
     """A bank model where one memory is in zero or one banks at any point"""
-    def __init__(self, radio):
-        self._radio = radio
+    def __init__(self, radio, name='Banks'):
+        super(BankModel, self).__init__(radio, name)
 
-    def get_num_banks(self):
-        """Returns the number of banks (should be callable without
-        consulting the radio"""
-        raise Exception("Not implemented")
-
-    def get_banks(self):
-        """Return a list of banks"""
-        raise Exception("Not implemented")
-
-    def add_memory_to_bank(self, memory, bank):
-        """Add @memory to @bank."""
-        raise Exception("Not implemented")
-
-    def remove_memory_from_bank(self, memory, bank):
-        """Remove @memory from @bank.
-        Shall raise exception if @memory is not in @bank."""
-        raise Exception("Not implemented")
-
-    def get_bank_memories(self, bank):
-        """Return a list of memories in @bank"""
-        raise Exception("Not implemented")
-
-    def get_memory_banks(self, memory):
-        """Returns a list of the banks that @memory is in"""
-        raise Exception("Not implemented")
-
-class BankIndexInterface:
-    """Interface for banks with index capabilities"""
+class MappingModelIndexInterface:
+    """Interface for mappings with index capabilities"""
     def get_index_bounds(self):
-        """Returns a tuple (lo,hi) of the minimum and maximum bank indices"""
-        raise Exception("Not implemented")
+        """Returns a tuple (lo,hi) of the min and max mapping indices"""
+        raise NotImplementedError()
 
-    def get_memory_index(self, memory, bank):
-        """Returns the index of @memory in @bank"""
-        raise Exception("Not implemented")
+    def get_memory_index(self, memory, mapping):
+        """Returns the index of @memory in @mapping"""
+        raise NotImplementedError()
 
-    def set_memory_index(self, memory, bank, index):
-        """Sets the index of @memory in @bank to @index"""
-        raise Exception("Not implemented")
+    def set_memory_index(self, memory, mapping, index):
+        """Sets the index of @memory in @mapping to @index"""
+        raise NotImplementedError()
 
-    def get_next_bank_index(self, bank):
-        """Returns the next available bank index in @bank, or raises
+    def get_next_mapping_index(self, mapping):
+        """Returns the next available mapping index in @mapping, or raises
         Exception if full"""
-        raise Exception("Not implemented")
-
+        raise NotImplementedError()
 
 class MTOBankModel(BankModel):
     """A bank model where one memory can be in multiple banks at once """
@@ -615,7 +643,15 @@ def console_status(status):
     import sys
 
     sys.stderr.write("\r%s" % status)
-    
+
+
+class RadioPrompts:
+    """Radio prompt strings"""
+    experimental = None
+    pre_download = None
+    pre_upload = None
+    display_pre_upload_prompt_before_opening_port = True
+
 
 BOOLEAN = [True, False]
 
@@ -652,6 +688,7 @@ class RadioFeatures:
         "valid_name_length"   : 0,
         "valid_cross_modes"   : [],
         "valid_dtcs_pols"     : [],
+        "valid_dtcs_codes"    : [],
         "valid_special_chans" : [],
 
         "has_sub_devices"     : BOOLEAN,
@@ -771,6 +808,8 @@ class RadioFeatures:
                   "Supported tone cross modes")
         self.init("valid_dtcs_pols", ["NN", "RN", "NR", "RR"],
                   "Supported DTCS polarities")
+        self.init("valid_dtcs_codes", list(DTCS_CODES),
+                  "Supported DTCS codes")
         self.init("valid_special_chans", [],
                   "Supported special channel names")
 
@@ -833,6 +872,13 @@ class RadioFeatures:
                                       mem.dtcs_polarity)
             msgs.append(msg)
 
+        if self.valid_dtcs_codes and \
+                mem.dtcs not in self.valid_dtcs_codes:
+            msg = ValidationError("DTCS Code %03i not supported" % mem.dtcs)
+        if self.valid_dtcs_codes and \
+                mem.rx_dtcs not in self.valid_dtcs_codes:
+            msg = ValidationError("DTCS Code %03i not supported" % mem.rx_dtcs)
+            
         if self.valid_duplexes and mem.duplex not in self.valid_duplexes:
             msg = ValidationError("Duplex %s not supported" % mem.duplex)
             msgs.append(msg)
@@ -893,7 +939,7 @@ class ValidationError(ValidationMessage):
     """A fatal error during memory validation"""
     pass
 
-class Radio:
+class Radio(object):
     """Base class for all Radio drivers"""
     BAUD_RATE = 9600
     HARDWARE_FLOW = False
@@ -918,6 +964,11 @@ class Radio:
         """Return a printable name for this radio"""
         return "%s %s" % (cls.VENDOR, cls.MODEL)
 
+    @classmethod
+    def get_prompts(cls):
+        """Return a set of strings for use in prompts"""
+        return RadioPrompts()
+
     def set_pipe(self, pipe):
         """Set the serial object to be used for communications"""
         self.pipe = pipe
@@ -941,9 +992,14 @@ class Radio:
         """Set the memory object @memory"""
         pass
 
-    def get_bank_model(self):
-        """Returns either a BankModel or None if not supported"""
-        return None
+    def get_mapping_models(self):
+        """Returns a list of MappingModel objects (or an empty list)"""
+        if hasattr(self, "get_bank_model"):
+            # FIXME: Backwards compatibility for old bank models
+            bank_model = self.get_bank_model()
+            if bank_model:
+                return [bank_model]
+        return []
 
     def get_raw_memory(self, number):
         """Return a raw string describing the memory at @number"""
@@ -1300,10 +1356,10 @@ def split_tone_encode(mem):
     "DTCS", 23, "N"
     """
 
-    txmode = txval = None
-    txpol = mem.dtcs_polarity[0]
-    rxmode = rxval = None
-    rxpol = mem.dtcs_polarity[1]
+    txmode = ''
+    rxmode = ''
+    txval = None
+    rxval = None
     
     if mem.tmode == "Tone":
         txmode = "Tone"
@@ -1325,5 +1381,14 @@ def split_tone_encode(mem):
         elif rxmode == "DTCS":
             rxval = mem.rx_dtcs
 
+    if txmode == "DTCS":
+        txpol = mem.dtcs_polarity[0]
+    else:
+        txpol = None
+    if rxmode == "DTCS":
+        rxpol = mem.dtcs_polarity[1]
+    else:
+        rxpol = None
+
     return ((txmode, txval, txpol),
             (rxmode, rxval, rxpol))
