@@ -14,7 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from chirp import chirp_common, yaesu_clone, directory
+from chirp import chirp_common, yaesu_clone, directory, errors
 from chirp import bitwise
 
 MEM_FORMAT = """
@@ -82,18 +82,18 @@ POWER_LEVELS = [chirp_common.PowerLevel("Hi", watts=5.00),
                 chirp_common.PowerLevel("L1", watts=0.05)]
 
 class VX5BankModel(chirp_common.BankModel):
-    def get_num_banks(self):
+    def get_num_mappings(self):
         return 5
 
-    def get_banks(self):
+    def get_mappings(self):
         banks = []
-        for i in range(0, self.get_num_banks()):
+        for i in range(0, self.get_num_mappings()):
             bank = chirp_common.Bank(self, "%i" % (i+1), "MG%i" % (i+1))
             bank.index = i
             banks.append(bank)
         return banks
 
-    def add_memory_to_bank(self, memory, bank):
+    def add_memory_to_mapping(self, memory, bank):
         _members = self._radio._memobj.bank_groups[bank.index].members
         _bank_used = self._radio._memobj.bank_used[bank.index]
         for i in range(0, len(_members)):
@@ -107,7 +107,7 @@ class VX5BankModel(chirp_common.BankModel):
                 return True
         raise Exception(_("{bank} is full").format(bank=bank))
 
-    def remove_memory_from_bank(self, memory, bank):
+    def remove_memory_from_mapping(self, memory, bank):
         _members = self._radio._memobj.bank_groups[bank.index].members
         _bank_used = self._radio._memobj.bank_used[bank.index]
 
@@ -128,7 +128,7 @@ class VX5BankModel(chirp_common.BankModel):
         if not remaining_members:
             _bank_used.current_member = 0xFF
 
-    def get_bank_memories(self, bank):
+    def get_mapping_memories(self, bank):
         memories = []
 
         _members = self._radio._memobj.bank_groups[bank.index].members
@@ -143,10 +143,11 @@ class VX5BankModel(chirp_common.BankModel):
             memories.append(self._radio.get_memory(member.channel+1))
         return memories
 
-    def get_memory_banks(self, memory):
+    def get_memory_mappings(self, memory):
         banks = []
-        for bank in self.get_banks():
-            if memory.number in [x.number for x in self.get_bank_memories(bank)]:
+        for bank in self.get_mappings():
+            if memory.number in [x.number for x in
+                                 self.get_mapping_memories(bank)]:
                     banks.append(bank)
         return banks
 
@@ -215,7 +216,7 @@ class VX5Radio(yaesu_clone.YaesuCloneModeRadio):
         mem.tmode = TMODES[_mem.tmode & 0x3] # masked so bad mems can be read
         if mem.duplex == "split":
             mem.offset = chirp_common.fix_rounded_step(mem.offset)
-        mem.rtone = mem.ctone = chirp_common.TONES[_mem.tone]
+        mem.rtone = mem.ctone = chirp_common.OLD_TONES[_mem.tone]
         mem.dtcs = chirp_common.DTCS_CODES[_mem.dtcs]
 
         mem.skip = _flg.pskip and "P" or _flg.skip and "S" or ""
@@ -262,7 +263,11 @@ class VX5Radio(yaesu_clone.YaesuCloneModeRadio):
         else:
             _mem.power = 0
         _mem.tmode = TMODES.index(mem.tmode)
-        _mem.tone = chirp_common.TONES.index(mem.rtone)
+        try:
+            _mem.tone = chirp_common.OLD_TONES.index(mem.rtone)
+        except ValueError:
+            raise errors.UnsupportedToneError(
+                ("This radio does not support tone %s" % mem.rtone))
         _mem.dtcs = chirp_common.DTCS_CODES.index(mem.dtcs)
 
         _flg.skip = mem.skip == "S"
