@@ -15,6 +15,7 @@
 
 from chirp import chirp_common, yaesu_clone, directory
 from chirp import bitwise
+from textwrap import dedent
 
 MEM_FORMAT = """
 #seekto 0x0611;
@@ -103,18 +104,18 @@ def _is220(freq):
 
 class VX7BankModel(chirp_common.BankModel):
     """A VX-7 Bank model"""
-    def get_num_banks(self):
+    def get_num_mappings(self):
         return 9
 
-    def get_banks(self):
+    def get_mappings(self):
         banks = []
-        for i in range(0, self.get_num_banks()):
+        for i in range(0, self.get_num_mappings()):
             bank = chirp_common.Bank(self, "%i" % (i+1), "MG%i" % (i+1))
             bank.index = i
             banks.append(bank)
         return banks
 
-    def add_memory_to_bank(self, memory, bank):
+    def add_memory_to_mapping(self, memory, bank):
         _members = self._radio._memobj.bank_members[bank.index]
         _bank_used = self._radio._memobj.bank_used[bank.index]
         for i in range(0, 48):
@@ -123,7 +124,7 @@ class VX7BankModel(chirp_common.BankModel):
                 _bank_used.in_use = 0x0000
                 break
 
-    def remove_memory_from_bank(self, memory, bank):
+    def remove_memory_from_mapping(self, memory, bank):
         _members = self._radio._memobj.bank_members[bank.index].members
         _bank_used = self._radio._memobj.bank_used[bank.index]
 
@@ -143,7 +144,7 @@ class VX7BankModel(chirp_common.BankModel):
         if not remaining_members:
             _bank_used.in_use = 0xFFFF
 
-    def get_bank_memories(self, bank):
+    def get_mapping_memories(self, bank):
         memories = []
 
         _members = self._radio._memobj.bank_members[bank.index].members
@@ -158,11 +159,11 @@ class VX7BankModel(chirp_common.BankModel):
             memories.append(self._radio.get_memory(number+1))
         return memories
 
-    def get_memory_banks(self, memory):
+    def get_memory_mappings(self, memory):
         banks = []
-        for bank in self.get_banks():
+        for bank in self.get_mappings():
             if memory.number in [x.number for x in
-                                 self.get_bank_memories(bank)]:
+                                 self.get_mapping_memories(bank)]:
                 banks.append(bank)
         return banks
 
@@ -183,6 +184,23 @@ class VX7Radio(yaesu_clone.YaesuCloneModeRadio):
     _block_lengths = [ 10, 8, 16193 ]
     _block_size = 8
 
+    @classmethod
+    def get_prompts(cls):
+        rp = chirp_common.RadioPrompts()
+        rp.pre_download = _(dedent("""\
+            1. Turn radio off.
+            2. Connect cable to MIC/SP jack.
+            3. Press and hold in the [MON-F] key while turning the radio on
+                 ("CLONE" will appear on the display).
+            4. <b>After clicking OK</b>, press the [BAND] key to send image."""))
+        rp.pre_upload = _(dedent("""\
+            1. Turn radio off.
+            2. Connect cable to MIC/SP jack.
+            3. Press and hold in the [MON-F] key while turning the radio on
+                 ("CLONE" will appear on the display).
+            4. Press the [V/M] key ("CLONE WAIT" will appear on the LCD)."""))
+        return rp
+        
     def _checksums(self):
         return [ yaesu_clone.YaesuChecksum(0x0592, 0x0610),
                  yaesu_clone.YaesuChecksum(0x0612, 0x0690),
